@@ -31,6 +31,7 @@ val run :
   ?output_type:output_type ->
   ?arg_store:arg_store ->
   ?exn_handler:exn_handler ->
+  ?socket:Unix.file_descr ->
   ?sockaddr:Unix.sockaddr ->
   ?port:int ->
   (cgi -> unit) -> unit
@@ -41,9 +42,10 @@ val run :
         Default: allow from all.
       @param output_type Default: [`Direct ""]
       @param arg_store Default: [`Automatic] for all arguments.
-      @param sockaddr The socket used by the web server to send the requests.
       @param exn_handler See {!Netcgi.exn_handler}.  Default: delegate
         all exceptions to the default handler.  
+      @param socket is a listening socket to use. Overrides [sockaddr]
+        and [port]
       @param sockaddr The sockaddr for listening. Overrides [port]
       @param port The port for listening. Needs to be specified if no
         [sockaddr] is passed.
@@ -69,6 +71,18 @@ val handle_request :
       open or must be closed.
    *)
 
+val handle_connection :
+  config -> output_type -> arg_store -> exn_handler ->
+  (cgi -> unit) -> Unix.file_descr ->
+    unit
+    (** [handle_connection config output_type arg_store eh f ?script_name fd]:
+        This is a lower-level interface that processes
+        exactly one connection [fd]. The descriptor is closed (even on
+        error).
+
+        The other arguments are just like for [run].
+    *)
+
 
 (* ---------------------------------------------------------------------- *)
 
