@@ -1,6 +1,7 @@
 #define HAVE_PTHREAD
 #define FANCY_PAGE_TABLES
-#undef HAVE_NEW_MODIFY
+#define HAVE_NEW_MODIFY
+#define WEAK_MODIFY
 #define HAVE_POSIX_SHM
 #define HAVE_POSIX_SEM_ANON
 #define HAVE_POSIX_SEM_NAMED
@@ -10,6 +11,7 @@
 #define HAVE_POSIX_MEMALIGN
 #define HAVE_SYSLOG
 #define HAVE_AT
+#define HAVE_MKFIFOAT
 #define HAVE_FCHDIR
 #define HAVE_FDOPENDIR
 #define HAVE_REALPATH
@@ -21,6 +23,7 @@
 #define HAVE_EVENTFD
 #define HAVE_TIMERFD
 #define HAVE_EPOLL
+#define HAVE_GETIFADDRS
 #define HAVE_GCC_COMPARE_AND_SWAP
 #define HAVE_GPROF
 #define HAVE_LOCALE
