@@ -1,4 +1,4 @@
-(* $Id: netsys_gssapi.mli 2195 2015-01-01 12:23:39Z gerd $ *)
+(* $Id$ *)
 
 (** GSS-API Definition *)
 
@@ -434,7 +434,7 @@ class type ['credential, 'name, 'context] poly_gss_api =
     method unwrap :
           't . context:'context ->
                input_message:message ->
-               output_message_preferred_type:[ `String | `Memory ] ->
+               output_message_preferred_type:[ `Bytes | `Memory ] ->
                out:( output_message:message ->
 		     conf_state:bool ->
 		     qop_state:qop ->
@@ -466,7 +466,7 @@ class type ['credential, 'name, 'context] poly_gss_api =
                conf_req:bool ->
                qop_req:qop ->
                input_message:message ->
-               output_message_preferred_type:[ `String | `Memory ] ->
+               output_message_preferred_type:[ `Bytes | `Memory ] ->
                out:( conf_state:bool ->
 		     output_message:message ->
 		     minor_status:minor_status ->
@@ -591,7 +591,7 @@ val string_of_flag : ret_flag -> string
 (** See RFC 2078, section 4 *)
 
 val nt_hostbased_service : oid
-  (** names like "service@hostname" *)
+  (** names like "service\@hostname" *)
 
 val nt_hostbased_service_alt : oid
   (** another OID for the same (RFC 1964 mentions it) *)
@@ -615,7 +615,7 @@ val nt_krb5_principal_name : oid
   (** a Kerberos 5 principal name (see {!Netgssapi_support} for parsers *)
 
 val parse_hostbased_service : string -> string * string
-  (** Returns ([service,host]) for "service@host". Fails if not parseable *)
+  (** Returns ([service,host]) for "service\@host". Fails if not parseable *)
 
 
 (** {2 Configuring clients} *)
