@@ -1,4 +1,4 @@
-(* $Id: generate.ml 2195 2015-01-01 12:23:39Z gerd $
+(* $Id$
  * ----------------------------------------------------------------------
  *
  *)
@@ -609,7 +609,7 @@ let output_xdr_type (mli:formatter) (f:formatter) (dl:xdr_def list) =
     (* typemap: maps xdr_name to bound type *)
 
   let get_type t = get_type_from_map typemap t in
-  let get_type_of_decl td = get_type td.decl_type in
+  (* let get_type_of_decl td = get_type td.decl_type in *)
 
   let generated_types = ref [] in
 
@@ -695,7 +695,8 @@ let output_xdr_type (mli:formatter) (f:formatter) (dl:xdr_def list) =
 	      fprintf f "@[<hv 2>Netxdr.X_direct(";
 	    output_type (!s :: rectypes) false t';
 	    if direct then
-	      fprintf f ",@ _read_%s,@ _write_%s,@ _size_%s)@]" !s !s !s;
+	      fprintf f ",@ _read_%s,@ _write_%s,@ _size_%s,@ _expand_%s)@]"
+                      !s !s !s !s;
 	    fprintf f ")@]";
 	  end
       | T_enum l ->
@@ -1418,7 +1419,7 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
     match direct_opt with
       | None -> ()
       | Some n -> 
-	  fprintf f "| Netxdr.XV_direct(X_%s x, _) -> x@ " n in
+	  fprintf f "| Netxdr.XV_direct(X_%s x, _, _) -> x@ " n in
 
   let generate_dest direct_opt var regname regconv =
     fprintf f "@[<hv 2>( match %s with@ " var;
@@ -1740,7 +1741,7 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
     fprintf f "@]@\n"
   in
 
-  let rec output_ofconv_for_type (name:string) (var:string) (t:xdr_type) =
+  let rec output_ofconv_for_type ctx (name:string) (var:string) (t:xdr_type) =
     (* Generates an expression converting the O'Caml value contained in the
      * variable with name var to the corresponding XDR value
      *)
@@ -1751,10 +1752,12 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
 	| T_opaque_fixed _
 	| T_opaque _
 	| T_opaque_unlimited ->
-	    fprintf f "(Netxdr.XV_opaque %s)" var
+	    fprintf
+              f "(Netxdr.XV_opaque (%s.Netxdr.ctx_copy_string %s))" ctx var
 	| T_string _
 	| T_string_unlimited ->
-	    fprintf f "(Netxdr.XV_string %s)" var
+	    fprintf
+              f "(Netxdr.XV_string (%s.Netxdr.ctx_copy_string %s))" ctx var
 	| T_mstring(_,_)
 	| T_mstring_unlimited _ ->
 	    fprintf f "(Netxdr.XV_mstring %s)" var
@@ -1763,14 +1766,14 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
 	    fprintf f "( match %s with@ " var;
 	    fprintf f "| None   -> Netxdr.xv_none@ ";
 	    fprintf f "| Some x -> @[<hv 2>Netxdr.xv_some@ ";
-	    output_ofconv_for_type name "x" t';
+	    output_ofconv_for_type ctx name "x" t';
 	    fprintf f "@]@]@ )";
 	| T_array_fixed(_,t') ->
-	    output_ofconv_for_array name  var t'
+	    output_ofconv_for_array ctx name  var t'
 	| T_array(_,t') ->
-	    output_ofconv_for_array name var t'
+	    output_ofconv_for_array ctx name var t'
 	| T_array_unlimited t' ->
-	    output_ofconv_for_array name var t'
+	    output_ofconv_for_array ctx name var t'
 	| T_int _ 
 	| T_uint _
 	| T_hyper _
@@ -1789,7 +1792,7 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
 	      try Hashtbl.find typenames !n
 	      with Not_found -> assert false
 	    in
-	    fprintf f "(_of_%s %s)" ocaml_n var
+	    fprintf f "(_xof_%s %s %s)" ocaml_n ctx var
 	| T_enum l ->
 	    fprintf f "@[<hv>";
 	    fprintf f "(match Netnumber.int32_of_int4 %s with@ " var;
@@ -1825,7 +1828,7 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
 	      (fun (i,d) ->
                  let v = sprintf "x%d" i in
 	         fprintf f "let %s = " v;
-		 output_ofconv_for_type name v d.decl_type;
+		 output_ofconv_for_type ctx name v d.decl_type;
                  fprintf f " in@;"
 	      )
 	      tdl;
@@ -1844,7 +1847,7 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
 		   let _xdr_n   = d.decl_symbol.xdr_name in
 		   fprintf f "  @[<hv 2>(";
 		   fprintf f "let x = %s.%s in@ " var ocaml_n;
-		   output_ofconv_for_type name "x" d.decl_type;
+		   output_ofconv_for_type ctx name "x" d.decl_type;
 		   fprintf f ")@];@ ";
 		 end
 	      )
@@ -1869,7 +1872,7 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
 		       output_any_int f discr_type (sign,n);
 		       fprintf f ",@ ";
 		       if have_x then
-			 output_ofconv_for_type name "x" d.decl_type
+			 output_ofconv_for_type ctx name "x" d.decl_type
 		       else
 			 fprintf f "Netxdr.XV_void";
 		       fprintf f "@])";
@@ -1885,7 +1888,7 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
 		       )
 		       else (
 			 if have_x then
-			   output_ofconv_for_type name "x" d.decl_type
+			   output_ofconv_for_type ctx name "x" d.decl_type
 			 else
 			   fprintf f "Netxdr.XV_void"
 		       );
@@ -1901,7 +1904,7 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
 		   conversion_netnumber_of_custom_int discr_type in
 		 if have_x then (
 		   fprintf f "let x = ";
-		   output_ofconv_for_type name "x" d.decl_type;
+		   output_ofconv_for_type ctx name "x" d.decl_type;
 		   fprintf f " in@ ";
 		   fprintf f "%s(%s discriminant, x)@]" 
 		     constr int_conversion;
@@ -1917,7 +1920,7 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
 		     | Some d ->
 			 fprintf f "let mkdefault x =@;<1 2>";
 			 fprintf f "@[<hv>";
-			 output_ofconv_for_type name "x" d.decl_type;
+			 output_ofconv_for_type ctx name "x" d.decl_type;
 			 fprintf f "@]";
 			 fprintf f " in@ ";
 		 )
@@ -1925,7 +1928,7 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
     );
     fprintf f "@]"
 
-  and output_ofconv_for_array name var t' =
+  and output_ofconv_for_array ctx name var t' =
     let t1 = get_type_from_map typemap t' in
     match t1 with
       | T_string _
@@ -1935,10 +1938,10 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
 	  fprintf f "@[<hv 2>Netxdr.XV_array@ ";
 	  fprintf f "@[<hv 2>(Array.map@ ";
 	  fprintf f "(fun x -> ";
-	  output_ofconv_for_type name "x" t';
+	  output_ofconv_for_type ctx name "x" t';
 	  fprintf f ")@ %s)@]@]" var
 	  
-  and output_ofconv_for_tuple name var tl =
+  and output_ofconv_for_tuple ctx name var tl =
     fprintf f "@[<hv 1>";
     fprintf f "(let (";
     let n = ref 0 in
@@ -1957,7 +1960,7 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
     List.iter
       (fun t ->
 	 fprintf f "  @[<hv 2>(";
-	 output_ofconv_for_type name ("x" ^ string_of_int !n) t;
+	 output_ofconv_for_type ctx name ("x" ^ string_of_int !n) t;
 	 fprintf f ");@]@ ";
 	 incr n
       )
@@ -2698,33 +2701,74 @@ let output_conversions (mli:formatter) (f:formatter) (dl:xdr_def list) =
       | _ -> false in
 
   let output_ofconv_declaration n t tname direct =
+    let decl_expand =
+      !Options.enable_direct && direct in
+    let do_direct_mapping =
+      !Options.enable_direct && direct && permit_direct t in
     (* MLI: *)
     fprintf mli "val _of_%s : %s -> Netxdr.xdr_value;;@\n" n tname;
+    fprintf mli "val _xof_%s : Netxdr.ctx -> %s -> Netxdr.xdr_value;;@\n" n tname;
     (* ML: *)
     fprintf f "@[<hv>";
     begin_decl();
-    fprintf f "_of_%s (x:%s) : Netxdr.xdr_value =@;<1 2>"
-      n
-      tname;
-    if !Options.enable_direct && direct && permit_direct t then (
+    fprintf f "_of_%s (x:%s) : Netxdr.xdr_value =@;<1 2>" n tname;
+    fprintf f "@[<hv>_xof_%s Netxdr.default_ctx x@]" n;
+    fprintf f "@]@\n";
+
+    fprintf f "@[<hv>";
+    begin_decl();
+    fprintf f "_xof_%s (ctx:Netxdr.ctx) (x:%s) : Netxdr.xdr_value =@;<1 2>"
+      n tname;
+    if do_direct_mapping then (
       fprintf f
-	"@[<hv>Netxdr.XV_direct(X_%s x, _sizeexpr_%s x)@]" tname n
+        "@[<v>";
+      fprintf f
+        "if ctx.Netxdr.ctx_direct then@ ";
+      fprintf f
+	"  Netxdr.XV_direct(X_%s x, _sizeexpr_%s x, _expand_%s)@ " tname n n;
+      fprintf f
+        "else@ ";
+      fprintf f
+        "  let ctx' = { ctx with Netxdr.ctx_direct = ctx.Netxdr.ctx_direct_sub } in@ ";
+      fprintf f
+        "  @[<hv>";
+      output_ofconv_for_type "ctx'" n "x" t;
+      fprintf f
+        "@]@ @]";
     )
     else
-      output_ofconv_for_type n "x" t;
-    fprintf f "@]@\n"
+      output_ofconv_for_type "ctx" n "x" t;
+    fprintf f "@]@\n";
+
+    if decl_expand then (
+      fprintf f "@[<hv>";
+      begin_decl();
+      fprintf f "_expand_%s (ex:exn) : Netxdr.xdr_value =@;<1 2>" n;
+      fprintf f "match ex with@ ";
+      fprintf f "| X_%s x ->" tname; 
+      fprintf f " _xof_%s Netxdr.expand_ctx x@ " n;
+      fprintf f "| _ -> raise Netxdr.Dest_failure@ ";
+      fprintf f "@]@\n";
+    )
   in
 
   let output_ofconv_tuple_declaration n tl tname =
     (* MLI: *)
     fprintf mli "val _of_%s : %s -> Netxdr.xdr_value;;@\n" n tname;
+    fprintf mli "val _xof_%s : Netxdr.ctx -> %s -> Netxdr.xdr_value;;@\n" n tname;
     (* ML: *)
     fprintf f "@[<hv>";
     begin_decl();
-    fprintf f "_of_%s (x:%s) : Netxdr.xdr_value =@;<1 2>"
+    fprintf f "_of_%s (x:%s) : Netxdr.xdr_value =@;<1 2>" n tname;
+    fprintf f "@[<hv>_xof_%s Netxdr.default_ctx x@]" n;
+    fprintf f "@]@\n";
+
+    fprintf f "@[<hv>";
+    begin_decl();
+    fprintf f "_xof_%s ctx (x:%s) : Netxdr.xdr_value =@;<1 2>"
       n
       tname;
-    output_ofconv_for_tuple n "x" tl;
+    output_ofconv_for_tuple "ctx" n "x" tl;
     fprintf f "@]@\n"
   in
 
@@ -3030,14 +3074,14 @@ let output_client (mli:formatter) (f:formatter) (dl:xdr_def list)
 	    fprintf f "@[<hv 2>";
 	    fprintf f "let %s client arg =@ " proc.proc_symbol.ocaml_name;
 	    (* fprintf f "assert(Rpc_client.program client == _program);@ "; *)
-	    fprintf f "_to_%s'res (U'C.unbound_sync_call client _program \"%s\" (_of_%s'arg arg))"
+	    fprintf f "_to_%s'res (U'C.unbound_sync_call client _program \"%s\" (_xof_%s'arg (U'C.xdr_ctx client) arg))"
 	      pvp proc.proc_symbol.xdr_name pvp;
 	    fprintf f "@]@ @ ";
 	    
 	    fprintf f "@[<hv 2>";
 	    fprintf f "let %s'async client arg pass_reply =@ " proc.proc_symbol.ocaml_name;
 	    (* fprintf f "assert(Rpc_client.program client == _program);@ "; *)
-	    fprintf f "U'C.unbound_async_call client _program \"%s\" (_of_%s'arg arg)@ "
+	    fprintf f "U'C.unbound_async_call client _program \"%s\" (_xof_%s'arg (U'C.xdr_ctx client) arg)@ "
 	      proc.proc_symbol.xdr_name pvp;
 	    fprintf f "  (fun g -> pass_reply (fun () -> _to_%s'res (g())))@ " pvp;
 	    fprintf f "@]@ @ "
@@ -3146,7 +3190,7 @@ let output_server (style:style)
 		   proc.proc_symbol.ocaml_name in
 	 fprintf f "@ (Rpc_server.Sync { @[<v>Rpc_server.sync_name = \"%s\";@ "
 	   proc.proc_symbol.xdr_name;
-	 fprintf f "Rpc_server.sync_proc = (fun x -> _of_%s'res (proc_%s (_to_%s'arg x)))@]});"
+	 fprintf f "Rpc_server.sync_proc = (fun srv x -> _xof_%s'res (Rpc_server.xdr_ctx srv)  (proc_%s (_to_%s'arg x)))@]});"
 	   pvp proc.proc_symbol.ocaml_name pvp;
       )
       vers.version_def;
@@ -3196,7 +3240,7 @@ let output_server (style:style)
 		   proc.proc_symbol.ocaml_name in
 	 fprintf f "@ (Rpc_server.Async { @[<v>Rpc_server.async_name = \"%s\";@ "
 	   proc.proc_symbol.xdr_name;
-	 fprintf f "Rpc_server.async_invoke = (fun s x -> proc_%s s (_to_%s'arg x) (fun y -> Rpc_server.reply s (_of_%s'res y)))@]});"
+	 fprintf f "Rpc_server.async_invoke = (fun srv s x -> proc_%s s (_to_%s'arg x) (fun y -> Rpc_server.reply s (_xof_%s'res (Rpc_server.xdr_ctx srv) y)))@]});"
 	  proc.proc_symbol.ocaml_name pvp pvp;
       )
       vers.version_def;
@@ -3257,7 +3301,7 @@ let output_server (style:style)
 		   proc.proc_symbol.ocaml_name in
 	 fprintf f "@ (Rpc_server.Sync { @[<v>Rpc_server.sync_name = \"%s\";@ "
 	   proc.proc_symbol.xdr_name;
-	 fprintf f "Rpc_server.sync_proc = (fun x -> _of_%s'res (proc_%s (_to_%s'arg x)))@]});"
+	 fprintf f "Rpc_server.sync_proc = (fun srv x -> _xof_%s'res (Rpc_server.xdr_ctx srv) (proc_%s (_to_%s'arg x)))@]});"
 	   pvp proc.proc_symbol.ocaml_name pvp;
       )
       vers.version_def;
@@ -3300,7 +3344,7 @@ let output_server (style:style)
 		   proc.proc_symbol.ocaml_name in
 	 fprintf f "@ (Rpc_server.Async { @[<v>Rpc_server.async_name = \"%s\";@ "
 	   proc.proc_symbol.xdr_name;
-	 fprintf f "Rpc_server.async_invoke = (fun s x -> proc_%s s (_to_%s'arg x) (fun y -> Rpc_server.reply s (_of_%s'res y)))@]});"
+	 fprintf f "Rpc_server.async_invoke = (fun srv s x -> proc_%s s (_to_%s'arg x) (fun y -> Rpc_server.reply s (_xof_%s'res (Rpc_server.xdr_ctx srv) y)))@]});"
 	  proc.proc_symbol.ocaml_name pvp pvp;
       )
       vers.version_def;
