@@ -1,4 +1,4 @@
-(* $Id: rpc_util.mli 2195 2015-01-01 12:23:39Z gerd $ *)
+(* $Id$ *)
 
 (** Utility functions *)
 
@@ -27,6 +27,6 @@ val string_of_value :
 
 val hex_dump_m :
       Netsys_mem.memory -> int -> int -> string
-val hex_dump_s :
-      string -> int -> int -> string
+val hex_dump_b :
+      Bytes.t -> int -> int -> string
    (** Format the contents as hex sequence *)
